function ls_generator_OsGen3(n,eig_lo,eig_up,save_name,tfocs_max_itr,...
                             xopt_order,num_rot,pcdm_max_itr,pssgb_max_itr)

addpath(genpath('solvers/'));

randn('state',250);
rand ('state',250);

% number of rows in A.
m = 2*n;
    
% Sparsity of xopt.
k = n/2^7;

% Generate singular values for matrix A.
S = eig_up*rand(n,1);
S = S + eig_lo;

% Givens rotation.
theta = 2*pi/10;
ct = cos(theta);
st = sin(theta);
R = [
     ct  -st
     st   ct
    ];
Rt = R';

tic;
%% First round of rotations.
Gt = kron(sparse(1:n/2,1:n/2,ones(n/2,1),n/2,n/2),sparse(Rt));

%% Second round of rotations
indices_i = 2:2:n;

rows_Gt_2 = zeros(2*n,1);
cols_Gt_2 = zeros(2*n,1);
vals_Gt_2 = zeros(2*n,1);

rows_Gt_2(1:n) = 1:n;
cols_Gt_2(1:n) = 1:n;
vals_Gt_2(1:n) = 1;

num_nnz = n;

for p=1:length(indices_i)-1
   i = indices_i(p);
   
   rows_Gt_2(num_nnz + 1:num_nnz+4) = [i; i;   i+1; i+1];
   cols_Gt_2(num_nnz + 1:num_nnz+4) = [i; i+1; i;   i+1];
   vals_Gt_2(num_nnz + 1:num_nnz+4) = ...
                        [Rt(1,1)-1; Rt(1,2); Rt(2,1); Rt(2,2)-1];
   
   num_nnz = num_nnz + 4;  
end

Gt_2 = sparse(rows_Gt_2(1:num_nnz),cols_Gt_2(1:num_nnz),vals_Gt_2(1:num_nnz));

% %% Third round of rotations.
% indices_i = 1:6:n;
% 
% rows_Gt_3 = zeros(2*n,1);
% cols_Gt_3 = zeros(2*n,1);
% vals_Gt_3 = zeros(2*n,1);
% 
% rows_Gt_3(1:n) = 1:n;
% cols_Gt_3(1:n) = 1:n;
% vals_Gt_3(1:n) = 1;
% 
% num_nnz = n;
% 
% for p=1:length(indices_i)-1
%    i = indices_i(p);
%    
%    rows_Gt_3(num_nnz + 1:num_nnz+4) = [i; i;   i+3; i+3];
%    cols_Gt_3(num_nnz + 1:num_nnz+4) = [i; i+3; i;   i+3];
%    vals_Gt_3(num_nnz + 1:num_nnz+4) = ...
%                         [Rt(1,1)-1; Rt(1,2); Rt(2,1); Rt(2,2)-1];
%    
%    num_nnz = num_nnz + 4;  
% end
% 
% Gt_3 = sparse(rows_Gt_3(1:num_nnz),cols_Gt_3(1:num_nnz),vals_Gt_3(1:num_nnz));

%% Combine all rotations and add an extra Gt rotation in between.
if num_rot == 2
    Gt = Gt*Gt_2;
end
if num_rot == 3
    Gt = Gt*Gt_2*Gt;
end
if num_rot == 4
    Gt = Gt*Gt_2*Gt*Gt_2;
end

%clear Gt_2 Gt_3;

% create matrix A.
A = [sparse(1:n,1:n,S,n,n)*Gt;sparse(m-n,n)];
toc;

% create optimal solution xopt.
temp = Gt'*(xopt_order./(S.^2));
[~, idx] = sort(abs(temp));
xopt = zeros(n,1);
xopt(idx(1:k/2)) = temp(idx(1:k/2));
xopt(idx(end:-1:end-k/2 + 1)) = temp(idx(end:-1:end-k/2 + 1));

% create noise.

sub_g = zeros(n,1);
sub_g(xopt ~= 0) = sign(xopt(xopt ~= 0));
sub_g(xopt == 0) = 1.8.*rand(sum(xopt == 0),1) - 0.9;

e_n = (Gt*sub_g)./S;
e_n_to_m = rand(m-n,1);

e = [e_n;e_n_to_m];

% Create b and b_n (noise version).
b = A*xopt;
b_n = b + e;

tau = 1.0;

[m,n] = size(A);

fvalOpt = tau*norm(xopt,1) + 0.5*norm(A*xopt-b_n)^2;
    
%% Call pdNCG.
options.n        = n;
options.A        = A;
options.b        = b_n;
options.tau      = tau;
options.problem  = 'ls';
options.tol      = 1.0e-4;
options.maxpdNCGitr = 50;
% options.c3       = 0.5;
% options.max_itr_bt = 1;
options.verbose  = 2;

tic;
[x_pdNCG, ~, out_pdNCG] = pdNCGinterface(options);
time_pdNCG = toc;    

%% Call the TFOCS solver
opts = struct('restart',-Inf,'tol',1e-13,'maxits',tfocs_max_itr,'alg','N83');
opts.errFcn = @(f,primal) toc; 
fvalpdNCG = tau*norm(x_pdNCG,1) + 0.5*norm(A*x_pdNCG-b_n)^2;
opts.stopFcn = @(f,primal) f <= fvalpdNCG;
    
tic;
[x_tfocs, out_tfocs] = tfocs( smooth_quad, {A, -b_n}, prox_l1( tau ), [], opts );
time_tfocs = toc;

%% Call pcdm
   
threads = 40;
omega = max(sum(A~=0,2));
p = randperm(n);      
                                                            
[fval_pcdm, cpptime_pcdm, cpptime_pcdm2, objvals_pcdm, cpuTimes_pcdm, x_pcdm] = large_scale_experiment(n, m, p, fvalOpt, fvalpdNCG, threads, omega, A, full(b_n), pcdm_max_itr, tau);

%% Call PSSgb
w_init = zeros(n,1);

lambdaVect = tau*[0;ones(n-1,1)];

funObj = @(w)SquaredError(w,A,b_n);

gOptions.maxIter = pssgb_max_itr;
gOptions.verbose = 1; % Set to 0 to turn off output
gOptions.corrections = 10; % Number of corrections to store for L-BFGS methods
gOptions.optTol = 1.0e-16;
gOptions.progTol = 1.0e-16;


tic;
[wPSSgb, out_pssgb] = L1General2_PSSgb(funObj,w_init,lambdaVect,gOptions,fvalpdNCG);
time_pssgb = toc;

%% Print info.
phi_ls = @(w) 0.5*norm(A*w-b_n,2)^2;
    
obj_tfocs = tau*norm(x_tfocs,1) + phi_ls(x_tfocs);
obj_pdNCG = tau*norm(x_pdNCG,1) + phi_ls(x_pdNCG);
obj_pcdm = tau*norm(x_pcdm,1) + phi_ls(x_pcdm);
obj_pssgb = tau*norm(wPSSgb,1) + phi_ls(wPSSgb);

fprintf('\n');
fprintf('Optimal objective %1.2e \n',fvalOpt);
fprintf('pdNCG, CPU time: %1.2e, Obj: %1.2e \n',time_pdNCG,obj_pdNCG);
fprintf('tfocs, CPU time: %1.2e, Obj: %1.2e \n',time_tfocs,obj_tfocs);
fprintf('pcdm, CPU time: %1.2e, Obj: %1.2e \n',cpptime_pcdm2,obj_pcdm);
fprintf('pssgb, CPU time: %1.2e, Obj: %1.2e \n',time_pssgb,obj_pssgb);

clf;

loglog(out_pdNCG.t,out_pdNCG.f,'b','LineWidth',4);
hold on;
loglog(out_tfocs.err,out_tfocs.f,'r--','LineWidth',4);
hold on;
loglog(cpuTimes_pcdm,objvals_pcdm,'g-.','LineWidth',4);
hold on;
loglog(out_pssgb.cputimes(1:out_pssgb.iters),out_pssgb.objvals(1:out_pssgb.iters),'black:','LineWidth',6);

xmax = max([out_pdNCG.t;out_tfocs.err;cpuTimes_pcdm;out_pssgb.cputimes(1:out_pssgb.iters)]);
xlim([1.0e-3,xmax]);

ymax = max([out_pdNCG.f;out_tfocs.f;objvals_pcdm;out_pssgb.objvals(1:out_pssgb.iters)]);
ymin = min([out_pdNCG.f;out_tfocs.f;objvals_pcdm;out_pssgb.objvals(1:out_pssgb.iters)]);
ylim([ymin,ymax]);

leg = legend('pdNCG','FISTA','PCDM','PSSgb');
set(leg,'FontSize',15,'Location','SouthWest');
legend('boxoff');
ylabel('\bf{f_\tau(x)}', 'FontSize', 18, 'Rotation', 90);
xlabel('\bf{wall clock time (seconds)}', 'FontSize', 18);
set(gca, 'FontSize', 13);

fprintf('\n');
fprintf('cond(A^TA): %1.15e\n', (max(S)/min(S))^2);
fprintf('Sparsity A: %1.15e\n', nnz(A)/(n*m));

small_S = S.^2 >= 0.1;
P_small_xopt = Gt(small_S,:)'*(Gt(small_S,:)*xopt);
fprintf('cond(x^*) for rho = 0.1: %1.15e\n', norm(xopt)/norm(P_small_xopt));

savefig([save_name,'.fig']);
save('-v7.3',[save_name,'.mat']);

clear all;