#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "mpi_gen.h"
#include "mpi_pdNCG.h"

#define PI 3.14159265358979323846  /* pi */

int main(int argc, char **argv)
{
    /* Cluster info */
    int size_p, p;
    char hostname[128];
    size_t len = 126;
    
    /* Maximum number of iterations */
    int maxpdNCGitr = 100;
    
    /* Maximum number of iterations for pcg */
    int maxpcgitr = 10000;
    
    /* Create instance for pdNCG solver */
    PdNCG pdNCG_inst;
    
    /* Create a generator instance */
    Generator st_gen;
    
    /* mpi stuff below */
//    MPI::Init ( argc, argv );
    MPI_Init(&argc,&argv);
    MPI_Comm comm = MPI_COMM_WORLD;
    
    MPI_Comm_rank(comm,&p);
//    p = MPI::COMM_WORLD.Get_rank ( );
    MPI_Comm_size(comm,&size_p);
//    size_p = MPI::COMM_WORLD.Get_size ( );
    
    long long N = pow(2,40);   /* number of columns in matrix A */
    long long m = N*2;         /* number of constralong ints */
    long long k = N/pow(2,10); /* number of nonzeros in optimal solution */
    
    /* Sizes of vectors in processors */
    long long n_p = N/size_p;
    long long m_p = m/size_p;
    long long k_p = k/size_p;
    
    printf("n_%d = %lld \n", p, n_p);
    printf("m_%d = %lld \n", p, m_p);
    printf("k_%d = %lld \n", p, k_p);
    
    /* Allocate part of optimal solution for each processor */
    double *xopt_p = new double[n_p];
    
    /* Allocate singular values on each processor */
    double *s_val_p = new double[n_p];
    
    /* Allocate right hand side b */
    double *b_noisy_p = new double[m_p];
    
    /* Bound for smallest and largest singular values of matrix A */
    double eig_lo = 1.0e-1;
    double eig_up = 1.0e+2;
    
    /* Allocate Given's matrix and define degrees theta */
    double theta = PI/4;
    
    double ct = cos(theta);
    double st = sin(theta);
    double G[2][2] =
    {
        {ct, -st},
        {st,  ct}
    };
    
    /* Smoothing parameter */
    double mu = 1.0e-4;
    
    /* Tolerance for pcg */
    double tolpcg = 1.0e-1;
    
    /* Tolerance for rellative error */
    double tol = 1.0e-3;
    
    /* Generate problem data */
    st_gen.strongly_convex(p, size_p, N, m, k, n_p, m_p, k_p, xopt_p, s_val_p, theta,
                           eig_lo, eig_up, b_noisy_p, G);
    
    if (p ==0)
        printf("Problem generation finished \n");
    
//    for (long long i = 0; i < m_p; i++)
//        printf("b_noisy_p_%d[%lld] = %e \n", p, i, b_noisy_p[i]);
    
    /* Allocate part of solution for each processor */
    double *x_p = new double[n_p];
    
    for (long long i = 0; i < n_p; i++)
        x_p[i] = 0;
    
    /* Allocate memory for dual variables for each processor */
    double *y_p = new double[n_p];
    
    for (long long i = 0; i < n_p; i++)
        y_p[i] = 0;
    
    pdNCG_inst.pdNCG(p, size_p, N, m, k, n_p, m_p, k_p, xopt_p, s_val_p,
                     b_noisy_p, G, x_p, y_p, maxpdNCGitr, maxpcgitr, mu, tolpcg, tol);
    
//    for (long long i = 0; i < n_p; i++)
//    {
//        printf("xopt_%d[%lld] = %e \n", p, i, xopt_p[i]);
//        printf("x_%d[%lld] = %e \n", p, i, x_p[i]);
//    }
    
    delete[] xopt_p;
    delete[] x_p;
    delete[] y_p;
    delete[] s_val_p;
    delete[] b_noisy_p;
    

    MPI_Finalize();
    exit(0);
}